///////////////////////////////////////////////////////////////////////////////
// $Revision: 1.2 $
///////////////////////////////////////////////////////////////////////////////
/*
  Options for rexx syntax expansion/indenting may be accessed from SLICK's
  file extension setup menu (CONFIG, "File extension setup...").

  The extension specific options is a string of five numbers separated
  with spaces with the following meaning:

    Position       Option
       1             Minimum abbreviation.  Defaults to 1.  Specify large
                     value to avoid abbreviation expansion.
       2             Keyword case.  Values may be 0,1, or 2 which correspond
                     to lower case, upper case, and capitalized.  Default
                     is 0.
       3             Not used.  If we decide to do an "End Style", we will use
                     this postion.(reserved)
       4             reserved.
       5             reserved.

   Used "A Practical Approach to Programming the Rexx Language" for set up

*/
#include 'slick.sh'
#include 'tagsdb.sh'
#define MODE_NAME 'REXX'
#define EXTENSION 'rexx'
#define IDENTIFIER_CHARS  'A-Za-z0-9_!?.'
#define VLXLEXERNAME  'REXX'

defload()
{
   setup_info='MN='MODE_NAME',TABS=+3,MA=1 74 1,':+
              'KEYTAB='MODE_NAME'-keys,WW=0,IWT=0,ST=0,IN=2,WC='IDENTIFIER_CHARS',LN='VLXLEXERNAME',CF=1,';
   //compile_info='0 cobol *;'
   compile_info=''
   syntax_info='3 1 1 0 0 1 0'
   be_info=''
   create_ext(kt_index,EXTENSION,'',MODE_NAME,setup_info,compile_info,
              syntax_info,be_info);
   // Find the setup for CMD.
   setup_index = find_index('def-setup-cmd', MISC_TYPE);
   parse name_info(setup_index) with 'MN=' mode_name ',';
   if (_modename_eq(mode_name,'rexx')) {
      _DeleteExtensionOptions('cmd');
   }
}

static _str word_case(s)
{
   parse name_info(p_index) with . . . scase .
   if ( scase==0 ) {
      return(lowcase(s)) /* Lower case language key words. */
   } else if ( scase==1 ) {
      return(upcase(s))    /* Upper case language key words. */
   }
   if (substr(s,1,1)=='') {
      i=verify(s,' '\t)
      if (!i) {
         return(s);
      }
   } else {
      i=1;
   }
   return(substr(s,1,i-1):+upcase(substr(s,i,1)):+lowcase(substr(s,i+1)))  /* Capitalize */
}

// ---rgf; 2002-09-21
// removes comments from a line, assuming that beginning of comment is on the line, closing block comments
// takes care of nesting comments and comments within strings and the like ...
static _str remove_comments(line) // rgf, should deal with nested comments and other
{
// _str tmpString="... remove_comments(): received: ["line"]...";
// say(tmpString);
// messageNwait(tmpString);

   // say("   line=["line"], eol="eol"...");
   _str new_line = "",        // new line to be returned
        strDelis = '"' :+ "'", // Rexx string delimiters
        quote="",             // quote used for string in hand
        commentOpenDeli = "/*",   // open block comment delimiter
        commentCloseDeli= "*/";   // close block comment delimiter

      // make sure to ignore comments, if part of a string
   int stringPos=1,               // start of a string ?
       lineCommentPos=1,                // start of a line comment ? ("--")
       commentOpenPos=1,                // start of a block comment ?
       commentClosePos=1,               // end of a block comment ?
       commentEndPos=1,                 // start of a block comment ?
       count=0;                         // count level of comment nesting

   int rgfloop=0; // counter to inhibit deadlocks, rgf, 2003-07-10

   while (line!="") {       // loop over line, removing comments
                                  // rgf, 2003-07-10
      if (++rgfloop>128) break;   // unlikely that comments are nested to levl 128 or higher, hence break

      if (lineCommentPos>0)   lineCommentPos=pos("--", line);              // start of a line comment ? ("--")
      if (commentOpenPos>0)   commentOpenPos=pos(commentOpenDeli, line);   // start of a block comment ?
      if (commentClosePos>0)  commentClosePos=pos(commentCloseDeli, line); // start of a block comment ?

            // deal with strings in line (e.g. comments could be part of a string or vice versa)
      if (stringPos>0) {
          stringPos=verify(line, strDelis, "M");  // is there a quoted string ?
            // get next quoted string, if before comments, move it to "new_line"
          if (stringPos>0 &&
                 (lineCommentPos>0 ? (stringPos<lineCommentPos): true) &&
                 (commentOpenPos>0 ? (stringPos<commentOpenPos): true) &&
                 (commentClosePos>0? (stringPos<commentClosePos): true) ){

             int oldPos=stringPos, endPos=stringPos;  // find ending Rexx-quote
             quote=substr(line, stringPos, 1);  // extract used quote
             while (true) {
                endPos=pos(quote, line, oldPos+1);    // find next quote

                if (endPos==0) {       // no ending/closing quote found
                   endPos=stringPos;   // error there, treat umatched quote as if it was a string by itself
                   break;
                }
                else
                {
                   if (substr(line,endPos+1,1) != quote) {  // found !
                      break;
                   }
                   oldPos=endPos+1;    // hmm, not found yet, quote is escaped
                   continue;
                }
             }

             // parse line with before ':q','r' +0 string line; // does not work, because of the Rexx quote escaping mechanism
             parse line with before =(stringPos) string =(endPos+1) line;

             new_line=new_line before string;      // add text and string to new_line
             continue;     // re-get positions
          }
      }

            // line-comment not part of a string or comment, then ignore rest of line
      if ( lineCommentPos>0 &&
           (commentOpenPos>0 ? (lineCommentPos<commentOpenPos): true) &&
           (stringPos>0      ? (lineCommentPos<stringPos): true))
      {
         parse line with line "--" .;  // remove line comment
         lineCommentPos=0;             // no more line-comments possible
         continue;                     // continue
      }


      if (lineCommentPos+commentOpenPos==0) {    // no comments left, use rest of line verbatim
         break;
      }


      // rgf, 2003-07-10
      // o.k. block open on line, but no block end on the same line left ...
      if (commentOpenPos>0 && commentClosePos==0) {
         parse line with before =(commentOpenPos) . ;
         new_line=strip(before);
         line="";
         break;
      }


            // a block comment open in hand, determine nestings, find closing block comment
      if ((commentOpenPos>0 && (stringPos==0 || commentOpenPos<stringPos))) {

            // closing block-comment, if so, ignore from beginning of line up to and including the closing block comment deli
         if (commentClosePos>0 && commentClosePos<commentOpenPos) {
            line=substr(line, commentClosePos+2);     // remove lead-in comment from line
            continue;                  // start from top, getting new positions, if any
         }

         count=1;       // nesting level of comment

         int tmpStart=commentOpenPos,
             tmpOpen, tmpClose;
         while (count>0) {    // determine nesting and comment ending position
            tmpOpen=pos(commentOpenDeli,   line, tmpStart+2);
            tmpClose=pos(commentCloseDeli, line, tmpStart+2);

            if (tmpOpen>0 && (tmpClose==0 || tmpOpen<tmpClose)) {       // a valid open comment deli ?
               count++;
               tmpStart=tmpOpen;
            }
            else if (tmpClose>0 && (tmpOpen==0 ||tmpClose<tmpOpen)) {   // a valid closing comment deli ?
               count--;
               tmpStart=tmpClose;
               commentClosePos=tmpClose;
            }
            else break;    // no comment delis left, break
         }
         if (commentClosePos>0) {
            parse line with before =(commentOpenPos) cmt =(commentClosePos+2) line;
            new_line=new_line before;
         }
         continue;
      }
   }

// tmpString="... returning: ["new_line line"]... ------------------------";
// say(tmpString);
// messageNwait(tmpString);
   return new_line line;   // return line without comments
}

/* --->
static _str remove_comments(line,eol)  // this probably will not work for ensted comments
{
   _str new_line = "";
   int i = 1;
   int start_of_string = 0;
   do {
      if (pos("/ *",line,i)) {  // if there a comment block on line
         new_line = new_line:+substr(line,i,pos("/ *",line,i)-start_of_string-1);
         if (pos("* /",line,i)) {
            i=pos("* /",line,i)+2;
         }
         else i=eol;
      }
      else {  // if there is not a comment block remaining on the line
         if (i==1) {   // there were no / * characters on the line
            i=eol;
            new_line = substr(line,1,eol);  // return up to -- (end of line comments)
         }
         else {
            new_line = new_line:+substr(line,i,eol-i);  //copy the last, non-comment, part of the line
            i=eol;  // leave this after the new_line extension of the last part of the line does not get moved
         }
      }
      start_of_string=i;
   } while (i < eol );
   return(new_line)
}

---> */


_command void rexx_enter() name_info(','VSARG2_CMDLINE|VSARG2_ICON|VSARG2_REQUIRES_EDITORCTL|VSARG2_READ_ONLY)
{
   if (ispf_common_enter()) return;
   parse name_info(_edit_window().p_index) with . expand . . be_style .
   if ( command_state() || p_window_state:=='I' ||
      p_SyntaxIndent<0 || p_indent_style!=INDENT_SMART ||
      _in_comment(1) ||
         rexx_expand_enter(p_SyntaxIndent,expand) ) {
       call_root_key(ENTER)
   } else if (_argument=='') {
      _undo 'S'
   }
}

/* This command is bound to the SPACE BAR key.  It looks at the text around */
/* the cursor to decide whether insert an expanded template.  If it does not, */
/* the root key table definition for the SPACE BAR key is called. */
_command void rexx_space() name_info(','VSARG2_CMDLINE|VSARG2_LASTKEY|VSARG2_REQUIRES_EDITORCTL)
{
   parse name_info(_edit_window().p_index) with . expand . . be_style .;
   // Since rexx strings can span multiple lines, must check if where are in a string
   if ( command_state() || ! expand || p_SyntaxIndent<0 ||
      _in_comment() || _clex_find(0,'g')==CFG_STRING ||
      rexx_expand_space(p_SyntaxIndent,be_style) ) {
      if ( command_state() ) {
         call_root_key(' ')
      } else {
         keyin(' ');
      }
   } else if (_argument=='') {
      _undo 'S'
   }
}

/* These constant strings have been defined to make the syntax
 expansion and indenting more data driven and to speed up
 determining whether special processing must be performed. There
 must be a space before and after each key word. */
#define EXPAND_WORDS '  '
#define ENTER_WORDS  ' do end if otherwise select '
/*
static _str space_words[]={'arg','by','call','do','else','exit','for','forever','if',
                           'interpret','iterate','label','leave','nop','options',
                           'otherwise','parse','procedure','program',
                           'pull','push','queue','say','select',
                           'signal','to','trace',
                           'type','when','while'}; */
static _str space_words[]={'address','arg','call','do','else','exit','if',
                           'interpret','iterate','leave','nop','numeric','options',
                           'otherwise','parse','procedure','pull','push','queue',
                           'say','select','signal','trace','when'};

/* Returns non-zero number if fall through to enter key required */
static _str rexx_expand_enter(syntax_indent,expand)
{
  status=0;
  /* Is last word begin key word or first word one of indent on enter */
  /* key words?. */
  get_line(line);
  /* strip comments. */
  parse lowcase(line) with last_word '/*';
  if ( lastpos('{;| (do|if|select|end)([ \t]|$)}',last_word,'','r') ) {
     last_word=substr(last_word,lastpos('S0'),lastpos('0'))
  }
  parse line with '[~ \t]','r' +0 word '[ \t]','r' rest
  /* Put first word of line into first_word variable. */
  first_word=lowcase(word);
  last_word=lowcase(strip(last_word));
  if ( pos(' 'last_word' ',ENTER_WORDS,1,'i') && expand){
     indent_on_enter(syntax_indent);
     if (last_word=='select') {
        replace_line(indent_string(p_col-1)word_case('when  then do'));
        insert_line(indent_string(p_col-1)word_case('end'));
        up();_end_line();
        for (i=1;i<9;++i) left();
     }else if (last_word=='end') {
        status=up();
        get_line(line);
        endstillthere=pos('end',line,1,'i');
        if (!status) down();
        if (endstillthere) {
           save_pos(p);
           status=_nocomment_search('do','@-i');
           if (!status) {
              get_line(line);
              first_non_blank=pos('~[ ]',line,1,'r');
           }else{
              first_non_blank=p_col;
           }
           restore_pos(p);
           p_col=first_non_blank;
           status=0;
        }else{
           status=1;
        }
     }
  } else {
     status=1;
  }
  return(status)

/* Returns non-zero number if fall through to space bar key required */
}
static _str rexx_expand_space(syntax_indent,be_style)
{
   /* Put first word of line in lower case into word variable. */
   get_line line
   line=strip(line,'T')
   orig_word=lowcase(strip(line))
   if ( p_col!=text_col(_rawText(line))+1 ) {
      return(1)
   }
   if (pos(' ',orig_word)/* && !pos('do',orig_word,1,'ir')*/) {
      parse orig_word with first second;
      if (lowcase(strip(first))=='do') {
         orig_word=second;
      }
   }
   aliasfilename='';
   word=min_abbrev3(orig_word,space_words,name_info(p_index),aliasfilename);
   if (aliasfilename==null) return(0);
   if (aliasfilename!=''&&word!='') {
      if (orig_word:==word && orig_word==get_alias(word,mult_line_info,1,aliasfilename)) {
         _insert_text(' ');
         return(0);
      }
      col=p_col-_rawLength(orig_word);
      if (col==1) {
         line_prefix='';
      }else{
         line_prefix=indent_string(col-1);
      }
      replace_line(line_prefix);
      p_col=col;
      return(expand_alias(word,'',aliasfilename));
   }else{
      /* Is the cursor not at the end of the line or the first word */
      /* not one of the SPACE BAR expansion key words. */
      //if ( ! pos(' 'word' ',SPACE_WORDS) ) {
      if ( word=='') return(1)    /* Fall through to space bar key. */
      /* Insert the appropriate template based on the key word. */
      if (!isinteger(word)) {
         line=substr(line,1,length(line)-length(orig_word)):+word
      }else{
         //expanding
         //DO i=45
      }
      width=text_col(_rawText(line),_rawLength(line)-_rawLength(word)+1,'i')-1;
      if ( word=='do' ) {
         replace_line(word_case(line)' ');
         insert_line(indent_string(width):+word_case('end'));
         up();
         _end_line();
      } else if ( word=='address' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='arg' ) {
      // replace_line(word_case(line)'()');
         replace_line(word_case(line)' ');
         _end_line();
      // left();
      } else if ( word=='call' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='drop' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='else' ) {
         replace_line(word_case(line' do'));
         insert_line('');
         insert_line(indent_string(width):+word_case('end'));
         up();_end_line();
         p_col=width+syntax_indent+1;
      } else if ( word=='exit' ) {
         replace_line(word_case(line)' ');
         _end_line();
    /*} else if ( word=='for' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='forever' ) {
         replace_line(word_case(line)' ');
         _end_line();
    */} else if ( word=='if' ) {
         replace_line(word_case(line)'  'word_case('then')' 'word_case('do'));
         insert_line(indent_string(width):+word_case('end'));
         up();_end_line();
         for (i=1;i<9;++i) left();
      } else if ( word=='interpret' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='iterate' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='leave' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='nop' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='numeric' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='options' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='otherwise' ) {
      // replace_line(word_case(line'  do'));
      // insert_line(indent_string(width)word_case('end'));
      // up();_end_line();
         replace_line(word_case(line)' ');
         _end_line();
      // for (i=1;i<4;++i) left();
      } else if ( word=='parse' ) {
      // replace_line(word_case(line)'  with');
         replace_line(word_case(line)' ');
         _end_line();
         for (i=1;i<6;++i) left();
      } else if ( word=='procedure' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='pull' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='push' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='queue' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='return' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='say' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='select' ) {
         replace_line(word_case(line)' ');
     //  insert_line(indent_string(width+syntax_indent):+word_case('end'));
         insert_line(indent_string(width):+word_case('end'));
         up();
         _end_line();
      } else if ( word=='signal' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='trace' ) {
         replace_line(word_case(line)' ');
         _end_line();
    /*} else if ( word=='while' ) {
         replace_line(word_case(line)' ');
         _end_line();
    */} else if ( word=='when' ) {
      //   replace_line(word_case(line)' ');
      //   _end_line();
         replace_line(indent_string(width)word_case('when  then do'));
         insert_line(indent_string(width+11)word_case('end'));
         up();_end_line();
         for (i=1;i<9;++i) left();
    /*} else if ( word=='by' ) {
         replace_line(word_case(line)' ');
         _end_line();
      } else if ( word=='to' ) {
         p=pos('do ',line,1,'i')
         prefix=substr(line,1,p+length('do ')-1);
         suffix=word_case(substr(line,p+length('do')))' 'word_case('by')' 1';
         line=prefix' = ':+suffix;
         replace_line(line);
         _end_line();
         for (i=1;i<11;++i) left();
    */}
      return 0
   }
}
int rexx_proc_search(_str &proc_name,int find_first)
{
   static _str cur_class_name;
   static _str cur_meth_name; // rgf
   static _str cur_rout_name; // rgf
   int status=0;
   if (find_first) {
      cur_class_name='';
      cur_meth_name='';
      cur_rout_name='';
      if (proc_name:=='') {
         fn="[a-zA-Z_!?][a-zA-Z_!.?0-9]#|['][^']*[']|[\"][^\"]*[\"]";
      }else{
         messageNwait("searching for specified function name="fn);
         fn=_escape_re_chars(proc_name);
      }
      //status=_nocomment_search('^(:b|)'fn'(:b|)\:(:b|)(procedure|)','@ri');  // Allen trying next line
      //status=_nocomment_search('(\:\:)(:b@)requires|(\:\:)(:b@)method|(\:\:)(:b@)class|\:(:b@)procedure','@ri');
      status=search('\:#', '>@riXcs');  //This finds one or more non-comment :'s and places cursor at the end of the matching characters.
      /* status=search('^(:b|)' :+ '{\:\:(:b|)(requires|method|routine|class):b|}' :+
                    '{'fn'}' :+ '(:b|){\:(:b|)(procedure|)|}', '@riXcs'); */
   }else{
      status=repeat_search();
   /* //status=_nocomment_search('^(:b|)'fn'(:b|)\:(:b|)(procedure|)','@ri');
   }
   if (status) {
      return status;
   }
   get_line(line);
   parse line with '(:b|)','r' proc_name ':'
   proc_name=strip(proc_name);
   return(0);
}  */
   }

   for (;;) {
      if (status) {return status;}
      /* _str tag_type=get_match_text(0);
      proc_name=get_match_text(1);
      _str proc_proc=get_match_text(2); */
      _str tag_type='';
      _str proc_proc='';
      _str proc_class='';
      _str container='';   // ---rgf
      _str rest_of_line; get_line(rest_of_line); // ---rgf

      // int logical_eol = length(rest_of_line);
      // if (pos("--",rest_of_line)) logical_eol = pos("--",rest_of_line)-1; //set eol to beginning of end line comment
      // rest_of_line = remove_comments(rest_of_line,logical_eol);

      rest_of_line = remove_comments(rest_of_line);   // rgf, 2002-09-21

      //int start=length(tag_type" "proc_name);
      //start=verify(rest_of_line, " \t", "M", (start==0)? 1 : start );
      //tmp_line=strip(substr(rest_of_line, (start==0) ? 1 : start )); // actually, all comments should be removed from this too !
      int flags=0;
      //
      // the current problem is handling quoted proc_names.
      //
      if (pos("::",rest_of_line)) {    // we are processing an Object Rexx statement
         parse rest_of_line with '::' tag_type proc_name left_overs ;
         if (substr(proc_name,1,1) =="'") {
            start_pos = pos("'",rest_of_line);
            end_pos = pos("'",rest_of_line,start_pos+1);
            proc_name = substr(rest_of_line,start_pos,end_pos-start_pos+1);
         }
         if (substr(proc_name,1,1) =='"') {
            start_pos = pos('"',rest_of_line);
            end_pos = pos('"',rest_of_line,start_pos+1);
            proc_name = substr(rest_of_line,start_pos,end_pos-start_pos+1);
         }
      }
      else {                          // we had better be processing classic Rexx statement
         parse rest_of_line with proc_name ':';
         proc_name = strip(proc_name);
         // don't set tag_type so the last case will be caught
      }
      switch (lowcase(strip(tag_type))) {
      case 'class':
         cur_class_name=proc_name;
         cur_meth_name='';    // ---rgf
         cur_rout_name='';    // ---rgf
         tag_type = 'class';

         // naive testing for various stuff method (ie. "public")
         if (pos('public', rest_of_line)>0) { flags=VS_TAGFLAG_public;    } // tag_type='friend'; }
         else                               { flags=VS_TAGFLAG_access | VS_TAGFLAG_package; } // say("class: ["proc_name"] NOT public!");  }
         break;

      case 'method':
         cur_meth_name=proc_name;
         cur_rout_name='';    // ---rgf
         // tag_type = 'proc';
         tag_type = 'func';

         switch (lowcase(proc_name) ) {
         case 'init':   flags=VS_TAGTYPE_constructor; tag_type='constr'; break;
         case 'uninit': flags=VS_TAGTYPE_destructor;  tag_type='destr'; break;
         }

         if (cur_class_name=='') { container='(float)'; flags=0; } // ---rgf
         else                    { container=cur_class_name ; flags=flags | VS_TAGFLAG_inclass; }    // ---rgf


         // naive testing for various stuff method (ie. "static")
         if (pos('class', rest_of_line)>0)      { flags |= VS_TAGFLAG_static; } // say("class found for:"proc_name); }
         if (pos('private', rest_of_line)>0)    { flags |= VS_TAGFLAG_private; }
         if (pos('protected', rest_of_line)>0)  { flags |= VS_TAGFLAG_protected; }
         if (pos('attribute', rest_of_line)>0)  { tag_type='prop'; } // attribute method
         else  // check for attributes, which must be defined in an EXPOSE statement right after the ::method-directive
         {
            typeless p;       // to save positional infos
            _str str;
            save_pos(p);      // save all positional infos
            for(;down()==0; ) // go down one line
            {
              get_line(str);

              parse lowcase(str) with first_word str "--" .; // parse up to excluding line comment
              if (str=="") continue;  // get next line

// say("--> first_word=["first_word"] on line:" p_line);
              if (first_word!="expose") break;  // leave loop
              parse str with str "/*" . ;     // make sure no other comment remaining
              for (; str!="";) {  // iterate over list of attributes
                 parse str with attrib str ;
// say("         found=["attrib"] on line:" p_line);
// how to add ??                 proc_name=tag_tree_compose_tag(attrib,cur_meth_name,"prop", flags );
              }
              // restore_pos(p);   // restore all positional infos
              // return(0);
            }

            restore_pos(p);   // restore all positional infos
         }

         break;

      case 'routine':
         cur_class_name="";
         cur_meth_name='';    // ---rgf
         cur_rout_name=proc_name;
         tag_type = 'func';

         if (pos('public', rest_of_line)>0) { flags=VS_TAGFLAG_public;    }
         else                               { flags=VS_TAGFLAG_protected; } // say("routine: ["proc_name"] NOT public!");}
         break;

      case 'requires':
         tag_type = 'include';
         break;

      //case '':
      default:
         //
         // Rony, I commented the following out because the :procedure statments would not get tagged with it there.
         //  I did not try to figure out what it does yet but the tag creation appears to work OK without it.
         //
         /* if (!pos(':',proc_proc) || first_char(proc_name)=='"' || first_char(proc_name)=="'") {
            status=repeat_search();
            continue;
         }  */
         tag_type='proc';
         if (cur_meth_name!='')      { container=cur_meth_name; tag_type="subproc" /*VS_TAGSEPARATOR_package*/; flags=flags|VS_TAGFLAG_inclass ; }
         else if (cur_rout_name!='') { container=cur_rout_name; tag_type="subproc"; flags=flags|VS_TAGFLAG_inclass; }

         break;
      }

// say("rexx_proc_search: tag_type="tag_type" name="proc_name" proc="proc_proc);
      //messageNwait("--> compose_tag: proc_name=["proc_name"], container=["container"], tag_type=["tag_type"]");
      proc_name=tag_tree_compose_tag(proc_name,container,tag_type, flags );

      //say('rexx_proc_search='proc_name);
      return(0);
   }

}
/**
 * Build the REXX tag file which contains all the built-in REXX
 * functions and documentation found in "rexx.tagdoc".
 *
 * @param tfindex Name index of standard REXX tag file
 * @return 0 on success, nonzero on error.
 */
int _rexx_MaybeBuildTagFile(int &tfindex)
{
   return ext_MaybeBuildTagFile(tfindex,'rexx','rexx',"REXX Run-Times");
}
/**
 * @see _c_fcthelp_get_start
 */
int _rexx_fcthelp_get_start(_str (&errorArgs)[],
                         boolean OperatorTyped,
                         boolean cursorInsideArgumentList,
                         int &FunctionNameOffset,
                         int &ArgumentStartOffset,
                         int &flags
                         )
{
   return(_c_fcthelp_get_start(errorArgs,OperatorTyped,
                               cursorInsideArgumentList,
                               FunctionNameOffset,
                               ArgumentStartOffset,flags));
}
/**
 * @see _c_fcthelp_get
 */
int _rexx_fcthelp_get(_str (&errorArgs)[],
                      VSAUTOCODE_ARG_INFO (&FunctionHelp_list)[],
                      boolean &FunctionHelp_list_changed,
                      int &FunctionHelp_cursor_x,
                      _str &FunctionHelp_HelpWord,
                      int FunctionNameStartOffset,
                      int flags
                      )
{
   return(_c_fcthelp_get(errorArgs,
                         FunctionHelp_list,FunctionHelp_list_changed,
                         FunctionHelp_cursor_x,
                         FunctionHelp_HelpWord,
                         FunctionNameStartOffset,flags));
}

